@@ -18,6 +18,4 @@ from andes.core.discrete import AntiWindup, Selector, Switcher, DeadBand  # NOQA
 from andes.core.block import Block, Washout, Lag,  LeadLag, Piecewise  # NOQA
 from andes.core.block import LagAntiWindup, LeadLagLimit  # NOQA
 
-from andes.core.config import Config  # NOQA
-
-from andes.core.triplet import JacTriplet  # NOQA
+from andes.core.common import JacTriplet, Config  # NOQA
