@@ -1,8 +1,11 @@
+from typing import Optional, Iterable, Union, List, Tuple
+
 from andes.core.var import Algeb, State
-from andes.core.param import dummify
-from typing import Optional, Iterable, Dict, Union
-from andes.core.discrete import Discrete, AntiWindup, LessThan
-from andes.core.triplet import JacTriplet
+from andes.core.discrete import AntiWindup, LessThan, Selector, HardLimiter
+from andes.core.common import JacTriplet
+from andes.core.common import ModelFlags, dummify
+from collections import OrderedDict
+import numpy as np
 
 
 class Block(object):
@@ -88,13 +91,9 @@ class Block(object):
         self.tex_name = tex_name if tex_name else name
         self.info = info
         self.owner = None
-        self.vars: Dict[str, Union[Algeb, State, Discrete]] = dict()
+        self.vars = OrderedDict()
         self.triplets = JacTriplet()
-        self.flags = dict(
-            f_num=False,        # True if the block defines `f_numeric`
-            g_num=False,        # True if the block defines `g_numeric`
-            j_num=False,        # True if the block defines `j_numeric`
-        )
+        self.flags = ModelFlags()  # f_num, g_num and j_num can be set
 
     def __setattr__(self, key, value):
         # handle sub-blocks by prepending self.name
@@ -255,6 +254,7 @@ class PIController(Block):
         The integral gain parameter instance
 
     """
+
     def __init__(self, u, ref, kp, ki, name=None, info=None):
         super(PIController, self).__init__(name=name, info=info)
 
@@ -325,17 +325,20 @@ class PIControllerNumeric(Block):
 
 class Gain(Block):
     r"""
-    Gain block ::
+    Gain block. ::
 
-        u -> K -> y
+             ┌───┐
+        u -> │ K │ -> y
+             └───┘
 
     Exports an algebraic output `y`.
     """
+
     def __init__(self, u, K, name=None, tex_name=None, info=None):
         super().__init__(name=name, tex_name=tex_name, info=info)
-        self.u = u
+        self.u = dummify(u)
         self.K = dummify(K)
-        self.enforce_tex_name((self.K, ))
+        self.enforce_tex_name((self.K,))
 
         self.y = Algeb(info='Gain output', tex_name='y')
         self.vars = {'y': self.y}
@@ -345,7 +348,7 @@ class Gain(Block):
         Implemented equation and the initial condition are
 
         .. math ::
-            y = K u
+            y = K u \\
             y^{(0)} = K u^{(0)}
 
         """
@@ -355,21 +358,25 @@ class Gain(Block):
 
 class Integrator(Block):
     r"""
-    Integrator block ::
+    Integrator block. ::
 
-        u -> K/s -> y
+             ┌──────┐
+        u -> │ K/sT │ -> y
+             └──────┘
 
-    Exports a differential variable `y`. The initial output is specified by `y0` and default to zero.
+    Exports a differential variable `y`.
+    The initial output is specified by `y0` and default to zero.
     """
 
-    def __init__(self, u, K, y0=0, name=None, tex_name=None, info=None):
+    def __init__(self, u, T, K, y0, name=None, tex_name=None, info=None):
         super().__init__(name=name, tex_name=tex_name, info=info)
-        self.u = u
+        self.u = dummify(u)
         self.K = dummify(K)
-        self.enforce_tex_name((self.K, ))
-        self.y0 = y0
+        self.T = dummify(T)
+        self.y0 = dummify(y0)
+        self.enforce_tex_name((self.K, self.T))
 
-        self.y = State(info='Integrator output', tex_name='y')
+        self.y = State(info='Integrator output', tex_name='y', t_const=self.T)
         self.vars = {'y': self.y}
 
     def define(self):
@@ -381,24 +388,73 @@ class Integrator(Block):
             y^{(0)} = 0
 
         """
-        self.y.v_str = self.y0
-        self.y.e_str = f'{self.K.name} * {self.u.name}'
+        self.y.v_str = f'{self.y0.name}'
+        self.y.e_str = f'{self.K.name} * ({self.u.name})'
+
+
+class IntegratorAntiWindup(Block):
+    r"""
+    Integrator block with anti-windup limiter. ::
+
+                   upper
+                  /¯¯¯¯¯
+             ┌──────┐
+        u -> │ K/sT │ -> y
+             └──────┘
+           _____/
+           lower
+
+    Exports a differential variable `y` and an AntiWindup `lim`.
+    The initial output must be specified through `y0`.
+    """
+
+    def __init__(self, u, T, K, y0, lower, upper, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
+        self.T = dummify(T)
+        self.K = dummify(K)
+        self.y0 = dummify(y0)
+        self.lower = dummify(lower)
+        self.upper = dummify(upper)
+        self.enforce_tex_name((self.K, self.T))
+
+        self.y = State(info='AW Integrator output', tex_name='y', t_const=self.T)
+
+        self.lim = AntiWindup(u=self.y, lower=self.lower, upper=self.upper, tex_name='lim',
+                              info='Limiter in integrator',
+                              )
+
+        self.vars = {'y': self.y, 'lim': self.lim}
+
+    def define(self):
+        r"""
+        Implemented equation and the initial condition are
+
+        .. math ::
+            \dot{y} = K u \\
+            y^{(0)} = 0
+
+        """
+        self.y.v_str = f'{self.y0.name}'
+        self.y.e_str = f'{self.K.name} * ({self.u.name})'
 
 
 class Washout(Block):
     r"""
-    Washout filter (high pass) block ::
+    Washout filter (high pass) block. ::
 
-                sK
-         u -> ------- -> y
-              1 + sT
+             ┌────────┐
+             │   sK   │
+        u -> │ ────── │ -> y
+             │ 1 + sT │
+             └────────┘
 
     Exports state `x` (symbol `x'`) and output algebraic variable `y`.
     """
 
     def __init__(self, u, T, K, name=None, tex_name=None, info=None):
         super().__init__(name=name, tex_name=tex_name, info=info)
-        self.u = u
+        self.u = dummify(u)
         self.T = dummify(T)
         self.K = dummify(K)
         self.enforce_tex_name((self.K, self.T))
@@ -414,14 +470,14 @@ class Washout(Block):
         Equations and initial values:
 
         .. math ::
-            T \dot{x'} &= (u - x) \\
-            T y &= K (u - x) \\
+            T \dot{x'} &= (u - x') \\
+            T y &= K (u - x') \\
             x'^{(0)} &= u \\
             y^{(0)} &= 0
 
         """
         self.x.v_str = f'{self.u.name}'
-        self.y.v_str = f'0'
+        self.y.v_str = '0'
 
         self.x.e_str = f'({self.u.name} - {self.name}_x)'
         self.y.e_str = f'{self.K.name} * ({self.u.name} - {self.name}_x) - {self.T.name} * {self.name}_y'
@@ -441,7 +497,8 @@ class WashoutOrLag(Washout):
         If True, ``sT`` will become 1, and the washout will become a low-pass filter.
         If False, functions as a regular Washout.
     """
-    def __init__(self, u, T, K, name, zero_out=False, tex_name=None, info=None):
+
+    def __init__(self, u, T, K, name=None, zero_out=True, tex_name=None, info=None):
         super().__init__(u, T, K, name=name, tex_name=tex_name, info=info)
         self.zero_out = zero_out
         self.LT = LessThan(K,
@@ -461,8 +518,8 @@ class WashoutOrLag(Washout):
         Equations and initial values:
 
         .. math ::
-            T \dot{x'} &= (u - x) \\
-            T y = z_0 K (u-x) + z_1 Tx
+            T \dot{x'} &= (u - x') \\
+            T y = z_0 K (u - x') + z_1 T x \\
             x'^{(0)} &= u \\
             y^{(0)} &= 0
 
@@ -481,13 +538,15 @@ class WashoutOrLag(Washout):
 
 class Lag(Block):
     r"""
-    Lag (low pass filter) transfer function block ::
+    Lag (low pass filter) transfer function. ::
 
-                K
-        u -> ------ -> x
-             1 + sT
+             ┌────────┐
+             │    K   │
+        u -> │ ────── │ -> y
+             │ 1 + sT │
+             └────────┘
 
-    Exports one state variable `x` as the output.
+    Exports one state variable `y` as the output.
 
     Parameters
     ----------
@@ -499,17 +558,18 @@ class Lag(Block):
         Input variable
 
     """
-    def __init__(self, u, T, K, name=None, info='Lag transfer function'):
-        super().__init__(name=name, info=info)
-        self.u = u
+
+    def __init__(self, u, T, K, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
         self.T = dummify(T)
         self.K = dummify(K)
 
         self.enforce_tex_name((self.K, self.T))
-        self.x = State(info='State in lag transfer function', tex_name="x'",
+        self.y = State(info='State in lag transfer function', tex_name="y",
                        t_const=self.T)
 
-        self.vars = {'x': self.x}
+        self.vars = {'y': self.y}
 
     def define(self):
         r"""
@@ -520,26 +580,29 @@ class Lag(Block):
 
         .. math ::
 
-            T \dot{x'} &= (Ku - x') \\
-            x'^{(0)} &= K u
+            T \dot{y} &= (Ku - y) \\
+            y^{(0)} &= K u
 
         """
-        self.x.v_str = f'{self.u.name} * {self.K.name}'
-        self.x.e_str = f'({self.K.name} * {self.u.name} - {self.name}_x)'
+        self.y.v_str = f'{self.u.name} * {self.K.name}'
+        self.y.e_str = f'({self.K.name} * {self.u.name} - {self.name}_y)'
 
 
 class LagAntiWindup(Block):
     r"""
-    Lag (low pass filter) transfer function block with an anti-windup limiter ::
+    Lag (low pass filter) transfer function block with an anti-windup limiter. ::
 
-                      /-- upper --
-                     K
-             u -> ------ -> x
-                  1 + sT
-        -- lower --/
+                     upper
+                   /¯¯¯¯¯¯
+             ┌────────┐
+             │    K   │
+        u -> │ ────── │ -> y
+             │ 1 + sT │
+             └────────┘
+           ______/
+           lower
 
-    Exports one state variable `x` as the output.
-    Exports one AntiWindup instance `lim`.
+    Exports one state variable `y` as the output and one AntiWindup instance `lim`.
 
     Parameters
     ----------
@@ -551,10 +614,11 @@ class LagAntiWindup(Block):
         Input variable
 
     """
-    def __init__(self, u, T, K, lower, upper, name=None,
-                 info='Lag transfer function with non-windup limiter'):
-        super().__init__(name=name, info=info)
-        self.u = u
+
+    def __init__(self, u, T, K, lower, upper,
+                 name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
         self.T = dummify(T)
         self.K = dummify(K)
 
@@ -563,11 +627,12 @@ class LagAntiWindup(Block):
 
         self.enforce_tex_name((self.T, self.K))
 
-        self.x = State(info='State in lag transfer function', tex_name="x'",
+        self.y = State(info='State in lag TF', tex_name="y",
                        t_const=self.T)
-        self.lim = AntiWindup(u=self.x, lower=self.lower, upper=self.upper, tex_name='lim')
+        self.lim = AntiWindup(u=self.y, lower=self.lower, upper=self.upper, tex_name='lim',
+                              info='Limiter in Lag')
 
-        self.vars = {'x': self.x, 'lim': self.lim}
+        self.vars = {'y': self.y, 'lim': self.lim}
 
     def define(self):
         r"""
@@ -578,21 +643,23 @@ class LagAntiWindup(Block):
 
         .. math ::
 
-            T \dot{x'} &= (Ku - x) \\
-            x'^{(0)} &= K u
+            T \dot{y} &= (Ku - y) \\
+            y^{(0)} &= K u
 
         """
-        self.x.v_str = f'{self.u.name} * {self.K.name}'
-        self.x.e_str = f'{self.name}_lim_zi * ({self.K.name} * {self.u.name} - {self.name}_x)'
+        self.y.v_str = f'{self.u.name} * {self.K.name}'
+        self.y.e_str = f'{self.K.name} * {self.u.name} - {self.name}_y'
 
 
 class Lag2ndOrd(Block):
     r"""
     Second order lag transfer function (low-pass filter) ::
 
-                     K
-        u -> ----------------- -> y
-             1 + sT1 + s^2 T2
+             ┌──────────────────┐
+             │         K        │
+        u -> │ ──────────────── │ -> y
+             │ 1 + sT1 + s^2 T2 │
+             └──────────────────┘
 
     Exports one two state variables (`x`, `y`), where `y` is the output.
 
@@ -611,7 +678,7 @@ class Lag2ndOrd(Block):
     def __init__(self, u, K, T1, T2, name=None, tex_name=None, info=None):
         super(Lag2ndOrd, self).__init__(name=name, tex_name=tex_name, info=info)
 
-        self.u = u
+        self.u = dummify(u)
         self.K = dummify(K)
         self.T1 = dummify(T1)
         self.T2 = dummify(T2)
@@ -651,24 +718,35 @@ class LeadLag(Block):
     r"""
     Lead-Lag transfer function block in series implementation ::
 
-             1 + sT1
-        u -> ------- -> y
-             1 + sT2
+             ┌───────────┐
+             │   1 + sT1 │
+        u -> │ K ─────── │ -> y
+             │   1 + sT2 │
+             └───────────┘
 
     Exports two variables: internal state `x` and output algebraic variable `y`.
 
+    Notes
+    -----
+    To allow zeroing out lead-lag as a pure gain, set ``zero_out`` to `True`.
+
     Parameters
     ----------
     T1 : BaseParam
         Time constant 1
     T2 : BaseParam
         Time constant 2
+    zero_out : bool
+        True to allow zeroing out lead-lag as a pass through (when T1=T2=0)
     """
-    def __init__(self, u, T1, T2, name=None, tex_name=None, info='Lead-lag transfer function'):
+
+    def __init__(self, u, T1, T2, K=1, zero_out=True, name=None, tex_name=None, info=None):
         super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
         self.T1 = dummify(T1)
         self.T2 = dummify(T2)
-        self.u = u
+        self.K = dummify(K)
+        self.zero_out = zero_out
 
         self.enforce_tex_name((self.T1, self.T2))
 
@@ -676,6 +754,15 @@ class LeadLag(Block):
         self.y = Algeb(info='Output of lead-lag', tex_name=r'y', diag_eps=1e-6)
         self.vars = {'x': self.x, 'y': self.y}
 
+        if self.zero_out is True:
+            self.LT1 = LessThan(T1, dummify(0), equal=True, enable=zero_out, tex_name='LT',
+                                cache=True, z0=1, z1=0)
+            self.LT2 = LessThan(T2, dummify(0), equal=True, enable=zero_out, tex_name='LT',
+                                cache=True, z0=1, z1=0)
+            self.x.discrete = (self.LT1, self.LT2)
+            self.vars['LT1'] = self.LT1
+            self.vars['LT2'] = self.LT2
+
     def define(self):
         r"""
 
@@ -687,37 +774,56 @@ class LeadLag(Block):
         .. math ::
 
             T_2 \dot{x'} &= (u - x') \\
-            T_2  y &= T_1  (u - x') + T_2  x' \\
-            x'^{(0)} &= y^{(0)} = u
+            T_2  y &= K T_1  (u - x') + K T_2  x' + E_2 \, , \text{where} \\
+            E_2 = &
+            \left\{\begin{matrix}
+            (y - K x') &\text{ if } T_1 = T_2 = 0 \& zero\_out=True \\
+            0& \text{ otherwise }
+            \end{matrix}\right. \\
+            x'^{(0)} & = u\\
+            y^{(0)} & = Ku\\
+
 
         """
         self.x.v_str = f'{self.u.name}'
         self.y.v_str = f'{self.u.name}'
 
         self.x.e_str = f'({self.u.name} - {self.name}_x)'
-        self.y.e_str = f'{self.T1.name} * ({self.u.name} - {self.name}_x) + ' \
-                       f'{self.name}_x * {self.T2.name} - ' \
+        self.y.e_str = f'{self.K.name} * {self.T1.name} * ({self.u.name} - {self.name}_x) + ' \
+                       f'{self.K.name} * {self.name}_x * {self.T2.name} - ' \
                        f'{self.name}_y * {self.T2.name}'
 
+        # when T1=T2=0, use equation `0 = y - Kx`
+        if self.zero_out is True:
+            self.y.e_str += f'+ {self.name}_LT1_z1 * {self.name}_LT2_z1 * ' \
+                            f'({self.name}_y - {self.K.name} * {self.name}_x)'
+
 
 class LeadLag2ndOrd(Block):
     r"""
     Second-order lead-lag transfer function block ::
 
-             1 + sT3 + s^2 T4
-        u -> ----------------- -> y
-             1 + sT1 + s^2 T2
+             ┌──────────────────┐
+             │ 1 + sT3 + s^2 T4 │
+        u -> │ ──────────────── │ -> y
+             │ 1 + sT1 + s^2 T2 │
+             └──────────────────┘
 
     Exports two internal states (`x1` and `x2`) and output algebraic variable `y`.
+
+    # TODO: instead of implementing `zero_out` using `LessThan` and an additional
+    term, consider correcting all parameters to 1 if all are 0.
+
     """
 
-    def __init__(self, u, T1, T2, T3, T4, name=None, tex_name=None, info='2nd-order lead-lag'):
+    def __init__(self, u, T1, T2, T3, T4, zero_out=False, name=None, tex_name=None, info=None):
         super(LeadLag2ndOrd, self).__init__(name=name, tex_name=tex_name, info=info)
-        self.u = u
+        self.u = dummify(u)
         self.T1 = dummify(T1)
         self.T2 = dummify(T2)
         self.T3 = dummify(T3)
         self.T4 = dummify(T4)
+        self.zero_out = zero_out
         self.enforce_tex_name((self.T1, self.T2, self.T3, self.T4))
 
         self.x1 = State(info='State #1 in 2nd order lead-lag', tex_name="x'", t_const=self.T2)
@@ -726,6 +832,21 @@ class LeadLag2ndOrd(Block):
 
         self.vars = {'x1': self.x1, 'x2': self.x2, 'y': self.y}
 
+        if self.zero_out is True:
+            self.LT1 = LessThan(T1, dummify(0), equal=True, enable=zero_out, tex_name='LT',
+                                cache=True, z0=1, z1=0)
+            self.LT2 = LessThan(T2, dummify(0), equal=True, enable=zero_out, tex_name='LT',
+                                cache=True, z0=1, z1=0)
+            self.LT3 = LessThan(T4, dummify(0), equal=True, enable=zero_out, tex_name='LT',
+                                cache=True, z0=1, z1=0)
+            self.LT4 = LessThan(T4, dummify(0), equal=True, enable=zero_out, tex_name='LT',
+                                cache=True, z0=1, z1=0)
+            self.x2.discrete = (self.LT1, self.LT2, self.LT3, self.LT4)
+            self.vars['LT1'] = self.LT1
+            self.vars['LT2'] = self.LT2
+            self.vars['LT3'] = self.LT3
+            self.vars['LT4'] = self.LT4
+
     def define(self):
         r"""
         Notes
@@ -735,7 +856,12 @@ class LeadLag2ndOrd(Block):
         .. math ::
             T_2 \dot{x}_1 &= u - x_2 - T_1 x_1 \\
             \dot{x}_2 &= x_1 \\
-            T_2 y &= T_2 x_2 + T_2 T_3 x_1 + T_4 (u - x_2 - T_1 x_1) \\
+            T_2 y &= T_2 x_2 + T_2 T_3 x_1 + T_4 (u - x_2 - T_1 x_1) + E_2 \, , \text{ where} \\
+            E_2 = &
+            \left\{\begin{matrix}
+            (y - x_2) &\text{ if } T_1 = T_2 = T_3 = T_4 = 0 \& zero\_out=True \\
+            0& \text{ otherwise }
+            \end{matrix}\right. \\
             x_1^{(0)} &= 0 \\
             x_2^{(0)} &= y^{(0)} = u
 
@@ -751,32 +877,39 @@ class LeadLag2ndOrd(Block):
         self.x2.v_str = f'{self.u.name}'
         self.y.v_str = f'{self.u.name}'
 
+        # when T1=T2=0, use equation `0 = y - Kx`
+        if self.zero_out is True:
+            self.y.e_str += f'+ {self.name}_LT1_z1*{self.name}_LT2_z1*{self.name}_LT3_z1*{self.name}_LT4_z1 * ' \
+                            f'({self.name}_y - {self.name}_x2)'
+
 
 class LeadLagLimit(Block):
     r"""
     Lead-Lag transfer function block with hard limiter (series implementation) ::
 
-                                ___upper___
-              1 + sT1          /
-        u ->  -------  -> ynl / -> y
-              1 + sT2        /
-                  __lower___/
+             ┌─────────┐          upper
+             │ 1 + sT1 │         /¯¯¯¯¯
+        u -> │ ─────── │ -> ynl / -> y
+             │ 1 + sT2 │  _____/
+             └─────────┘  lower
 
     Exports four variables: state `x`, output before hard limiter `ynl`, output `y`, and AntiWindup `lim`.
 
     """
-    def __init__(self, u, T1, T2, lower, upper, name=None, info='Lead-lag transfer function'):
-        super().__init__(name=name, info=info)
-        self.T1 = T1
-        self.T2 = T2
-        self.u = u
+
+    def __init__(self, u, T1, T2, lower, upper,
+                 name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
+        self.T1 = dummify(T1)
+        self.T2 = dummify(T2)
         self.lower = lower
         self.upper = upper
         self.enforce_tex_name((self.T1, self.T2))
 
-        self.x = State(info='State in lead-lag transfer function', tex_name="x'", t_const=self.T2)
-        self.ynl = Algeb(info='Output of lead-lag transfer function before limiter', tex_name=r'y_{nl}')
-        self.y = Algeb(info='Output of lead-lag transfer function after limiter', tex_name=r'y',
+        self.x = State(info='State in lead-lag TF', tex_name="x'", t_const=self.T2)
+        self.ynl = Algeb(info='Output of lead-lag TF before limiter', tex_name=r'y_{nl}')
+        self.y = Algeb(info='Output of lead-lag TF after limiter', tex_name=r'y',
                        diag_eps=1e-6)
         self.lim = AntiWindup(u=self.ynl, lower=self.lower, upper=self.upper)
 
@@ -812,28 +945,189 @@ class LeadLagLimit(Block):
                        f'{self.name}_y'
 
 
-class Piecewise(Block):
+class HVGate(Block):
     """
-    Piecewise block.
+    High Value Gate. Outputs the maximum of two inputs. ::
 
-    This block takes a list of N points, [x0, x1, ...x_{n-1}] and a list of N+1 functions [fun0, ..., fun_n].
-    Inputs in each range (xk, x_{k+1}] applies its corresponding function `fun_k`. The last range (x_{n-1},
-    +inf) applies the last function `fun_n`.
+              ┌─────────┐
+        u1 -> │ HV Gate │
+              │         │ ->  y
+        u2 -> │  (MAX)  │
+              └─────────┘
+
+    """
+
+    def __init__(self, u1, u2, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u1 = dummify(u1)
+        self.u2 = dummify(u2)
+        self.enforce_tex_name((u1, u2))
+
+        self.sl = Selector(self.u1, self.u2, fun=np.maximum.reduce,
+                           info='HVGate Selector',
+                           )
+
+        self.y = Algeb(info='HVGate output', tex_name='y', discrete=self.sl)
+        self.vars = {'y': self.y, 'sl': self.sl}
+
+    def define(self):
+        """
+        Implemented equations and initial conditions
+
+        .. math ::
+
+            0 = s_0^{sl} u_1 + s_1^{sl} u_2 - y
+            y_0 = maximum(u_1, u_2)
+
+        Notes
+        -----
+        In the implementation, one should not use ::
+
+            self.y.v_str = f'maximum({self.u1.name}, {self.u2.name})',
+
+        because SymPy processes this equation to `{self.u1.name}`.
+        Not sure if this is a bug or intended.
+
+        """
+        self.y.v_str = f'{self.name}_sl_s0*{self.u1.name} + {self.name}_sl_s1*{self.u2.name}'
+        self.y.e_str = f'{self.name}_sl_s0*{self.u1.name} + {self.name}_sl_s1*{self.u2.name} - ' \
+                       f'{self.name}_y'
+
+
+class LVGate(Block):
+    """
+    Low Value Gate. Outputs the minimum of the two inputs. ::
+
+              ┌─────────┐
+        u1 -> │ LV Gate |
+              │         | ->  y
+        u2 -> │  (MIN)  |
+              └─────────┘
+
+    """
+
+    def __init__(self, u1, u2, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u1 = dummify(u1)
+        self.u2 = dummify(u2)
+        self.enforce_tex_name((u1, u2))
+
+        self.y = Algeb(info='LVGate output', tex_name='y')
+        self.sl = Selector(self.u1, self.u2, fun=np.minimum.reduce,
+                           info='LVGate Selector',
+                           )
+
+        self.vars = {'y': self.y, 'sl': self.sl}
+
+    def define(self):
+        """
+        Implemented equations and initial conditions
+
+        .. math ::
+
+            0 = s_0^{sl} u_1 + s_1^{sl} u_2 - y
+            y_0 = minimum(u_1, u_2)
+
+        Notes
+        -----
+        Same problem as `HVGate` as `minimum` does not sympify correctly.
+
+        """
+        self.y.v_str = f'{self.name}_sl_s0*{self.u1.name} + {self.name}_sl_s1*{self.u2.name}'
+        self.y.e_str = f'{self.name}_sl_s0*{self.u1.name} + {self.name}_sl_s1*{self.u2.name} - ' \
+                       f'{self.name}_y'
+
+
+class GainLimiter(Block):
+    """
+    Gain followed by a limiter.
+
+    Exports the limited output `y`, unlimited output `x`, and HardLimiter `lim`. ::
+
+             ┌─────┐         upper
+             │     │        /¯¯¯¯¯
+        u -> │  K  │ -> x  / -> y
+             │     │ _____/
+             └─────┘ lower
 
     Parameters
     ----------
-    points : list
+    u : str, BaseVar
+        Input variable, or an equation string for constructing an anonymous variable
+
+    """
+
+    def __init__(self, u, K, upper, lower, no_upper=False, no_lower=False,
+                 name=None, tex_name=None, info=None):
+        Block.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.u = dummify(u)
+        self.K = dummify(K)
+        self.upper = upper
+        self.lower = lower
+
+        if (no_upper and no_lower) is True:
+            raise ValueError("no_upper or no_lower cannot both be True")
+
+        self.no_lower = no_lower
+        self.no_upper = no_upper
+
+        self.x = Algeb(info='Gain output before limiter', tex_name='x')
+        self.y = Algeb(info='Gain output after limiter', tex_name='y')
+
+        self.lim = HardLimiter(u=self.x, lower=self.lower, upper=self.upper,
+                               no_upper=no_upper, no_lower=no_lower,
+                               tex_name='lim')
+
+        self.vars = {'lim': self.lim, 'x': self.x, 'y': self.y}
+
+    def define(self):
+        """
+        TODO: write docstring
+        """
+        self.x.v_str = f'{self.K.name} * ({self.u.name})'
+        self.x.e_str = f'{self.K.name} * ({self.u.name}) - {self.name}_x'
+
+        self.y.e_str = f'{self.name}_x * {self.name}_lim_zi'
+        self.y.v_str = f'{self.name}_x * {self.name}_lim_zi'
+
+        if not self.no_upper:
+            self.y.e_str += f' + {self.name}_lim_zu*{self.upper.name}'
+            self.y.v_str += f' + {self.name}_lim_zu*{self.upper.name}'
+        if not self.no_lower:
+            self.y.e_str += f' + {self.name}_lim_zl*{self.lower.name}'
+            self.y.v_str += f' + {self.name}_lim_zl*{self.lower.name}'
+
+        self.y.e_str += f' - {self.name}_y'
+
+
+class Piecewise(Block):
+    """
+    Piecewise block. Outputs an algebraic variable `y`.
+
+    This block takes a list of N points, [x0, x1, ...x_{n-1}] to define N+1 ranges,
+    namely (-inf, x0), (x0, x1), ..., (x_{n-1}, +inf).
+    and a list of N+1 functions [fun0, ..., fun_n].
+
+    Inputs that fall within each range applies the corresponding function.
+    The first range (-inf, x0) applies `fun_0`, and
+    the last range (x_{n-1}, +inf) applies the last function `fun_n`.
+
+    Parameters
+    ----------
+    points : list, tuple
         A list of piecewise points. Need to be provided in the constructor function.
-    funs : list
+    funs : list, tuple
         A list of strings for the piecewise functions. Need to be provided in the overloaded `define` function.
     """
-    def __init__(self, u, points: list, funs: list, name=None, tex_name=None, info=None):
+
+    def __init__(self, u, points: Union[List, Tuple], funs: Union[List, Tuple],
+                 name=None, tex_name=None, info=None):
         super().__init__(name=name, tex_name=tex_name, info=info)
         self.u = u
         self.points = points
         self.funs = funs
 
-        self.y = Algeb(info='Output of piecewise function', tex_name='y')
+        self.y = Algeb(info='Output of piecewise', tex_name='y')
         self.vars = {'y': self.y}
 
     def define(self):
@@ -846,7 +1140,7 @@ class Piecewise(Block):
         i = 0
         for i in range(len(self.points)):
             args.append(f'({self.funs[i]}, {self.u.name} <= {self.points[i]})')
-        args.append(f'({self.funs[i+1]}, True)')
+        args.append(f'({self.funs[i + 1]}, True)')
 
         args_comma = ', '.join(args)
         pw_fun = f'Piecewise({args_comma})'
