@@ -2,47 +2,12 @@ from typing import Optional, Union, Callable, List, Tuple
 
 import math
 import logging
+
 from andes.models.group import GroupBase
 from andes.shared import np
 logger = logging.getLogger(__name__)
 
 
-class DummyValue(object):
-    """
-    Class for converting a scalar value to a dummy parameter with `name` and `tex_name` fields.
-
-    A DummyValue object can be passed to Block, which utilizes the `name` field to dynamically generate equations.
-
-    Notes
-    -----
-    Pass a numerical value to the constructor for most use cases, especially when passing as a v-provider.
-    """
-    def __init__(self, value):
-        self.name = value
-        self.tex_name = value
-        self.v = value
-
-
-def dummify(param):
-    """
-    Dummify scalar parameter and return a DummyValue object. Do nothing for BaseParam instances.
-
-    Parameters
-    ----------
-    param : float, int, BaseParam
-        parameter object or scalar value
-
-    Returns
-    -------
-    DummyValue(param) if param is a scalar; param itself, otherwise.
-
-    """
-    if isinstance(param, (int, float)):
-        return DummyValue(param)
-    else:
-        return param
-
-
 class BaseParam(object):
     """
     The base parameter class.
@@ -172,7 +137,7 @@ class BaseParam(object):
     @property
     def n(self):
         """Return the count of elements in the value array."""
-        return len(self.v) if self.v else 0
+        return len(self.v)
 
 
 class DataParam(BaseParam):
@@ -207,6 +172,9 @@ class IdxParam(BaseParam):
                 ...
                 self.bus = IdxParam(model='Bus')
 
+    Notes
+    -----
+    This will be useful when, for example, one connects two TGs to one SynGen.
     """
     def __init__(self,
                  default: Optional[Union[float, str, int]] = None,
@@ -215,12 +183,23 @@ class IdxParam(BaseParam):
                  info: Optional[str] = None,
                  unit: Optional[str] = None,
                  mandatory: bool = False,
+                 unique: bool = False,
                  export: bool = True,
                  model: Optional[str] = None):
         super().__init__(default=default, name=name, tex_name=tex_name, info=info, unit=unit, mandatory=mandatory,
                          export=export)
+        self.property['unique'] = unique
         self.model = model  # must be a `Model` name for building BackRef - Not checked yet
 
+    def add(self, value=None):
+        if self.get_property('unique'):
+            if value in self.v:
+                logger.error('Your input data may be inconsistent.')
+                raise IndexError(f'Unique parameter {self.owner.class_name}.{self.name} '
+                                 f'contains duplicate value <{value}>.')
+
+        super().add(value)
+
 
 class NumParam(BaseParam):
     """
@@ -251,8 +230,14 @@ class NumParam(BaseParam):
 
     Other Parameters
     ----------------
+    Sn : str
+        Name of the parameter for the device base power.
+    Vn : str
+        Name of the parameter for the device base voltage.
     non_zero : bool
         True if this parameter must be non-zero
+    positive: bool
+        True if this parameter must be positive
     mandatory : bool
         True if this parameter must not be None
     power : bool
@@ -285,6 +270,7 @@ class NumParam(BaseParam):
     dc_voltage : bool
         True if the parameter is a DC voltage pu quantity under
         device base
+
     """
 
     def __init__(self,
@@ -294,7 +280,10 @@ class NumParam(BaseParam):
                  info: Optional[str] = None,
                  unit: Optional[str] = None,
                  vrange: Optional[Union[List, Tuple]] = None,
+                 Sn: str = 'Sn',
+                 Vn: str = 'Vn',
                  non_zero: bool = False,
+                 positive: bool = False,
                  mandatory: bool = False,
                  power: bool = False,
                  ipower: bool = False,
@@ -312,6 +301,7 @@ class NumParam(BaseParam):
                                        unit=unit, export=export)
 
         self.property = dict(non_zero=non_zero,
+                             positive=positive,
                              mandatory=mandatory,
                              power=power,
                              ipower=ipower,
@@ -324,6 +314,8 @@ class NumParam(BaseParam):
                              dc_current=dc_current,
                              dc_voltage=dc_voltage)
 
+        self.Sn = Sn
+        self.Vn = Vn
         self.pu_coeff = np.ndarray([])
         self.vin = None  # values from input
         self.vrange = vrange
@@ -355,7 +347,12 @@ class NumParam(BaseParam):
 
         # check for non-zero
         if value == 0.0 and self.get_property('non_zero'):
-            logger.debug(f'Non-zero parameter {self.owner.class_name}.{self.name} corrected to {self.default}')
+            logger.warning(f'Non-zero parameter {self.owner.class_name}.{self.name} corrected to {self.default}')
+            value = self.default
+
+        # check for positive
+        if value is not None and value <= 0.0 and self.get_property('positive'):
+            logger.warning(f'Positive parameter {self.owner.class_name}.{self.name} corrected to {self.default}')
             value = self.default
 
         super(NumParam, self).add(value)
@@ -508,12 +505,30 @@ class ExtParam(NumParam):
                  model: str,
                  src: str,
                  indexer=None,
+                 dtype=float,
+                 allow_none=False,
+                 default=0.0,
                  **kwargs):
-        super(ExtParam, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.model = model
         self.src = src
         self.indexer = indexer
+        self.dtype = dtype
         self.parent_model = None   # parent model instance
+        self.allow_none = allow_none
+        self.default = default
+
+    def add(self, value=None):
+        """
+        ExtParam has an empty `add` method.
+        """
+        pass
+
+    def restore(self):
+        """
+        ExtParam has an empty `restore` method
+        """
+        pass
 
     def link_external(self, ext_model):
         """
@@ -532,19 +547,25 @@ class ExtParam(NumParam):
 
             # TODO: the three `get` calls below is a bit inefficient - same loops for three times
             try:
-                self.v = ext_model.get(src=self.src, idx=self.indexer.v, attr='v')
+                self.v = ext_model.get(src=self.src, idx=self.indexer.v, attr='v',
+                                       allow_none=self.allow_none, default=self.default)
             except IndexError:
                 pass
 
             try:
-                self.vin = ext_model.get(src=self.src, idx=self.indexer.v, attr='vin')
-                self.pu_coeff = ext_model.get(src=self.src, idx=self.indexer.v, attr='vin')
+                self.vin = ext_model.get(src=self.src, idx=self.indexer.v, attr='vin',
+                                         allow_none=self.allow_none, default=self.default)
+                self.pu_coeff = ext_model.get(src=self.src, idx=self.indexer.v, attr='vin',
+                                              allow_none=self.allow_none, default=self.default)
             except KeyError:  # idx param without vin
                 pass
             except TypeError:  # vin or pu_coeff is None
                 pass
 
         else:
+            if self.allow_none:
+                raise NotImplementedError(f"{self.name}: allow_none not implemented for Model")
+
             parent_instance = ext_model.__dict__[self.src]
             self.property = dict(parent_instance.property)
 
@@ -564,3 +585,11 @@ class ExtParam(NumParam):
                 self.pu_coeff = parent_instance.pu_coeff[uid]
             except KeyError:
                 pass
+
+    def to_array(self):
+        """
+        Convert to array when d_type is not str
+        """
+        if self.dtype == str:
+            return
+        NumParam.to_array(self)
