@@ -136,7 +136,7 @@ def parse(system):
     return True
 
 
-def dump(system, output_format, full_path=None, overwrite=False):
+def dump(system, output_format, full_path=None, overwrite=False, **kwargs):
     """
     Dump the System data into the requested output format.
 
@@ -153,14 +153,15 @@ def dump(system, output_format, full_path=None, overwrite=False):
         True if successful; False otherwise.
     """
     if system.files.no_output:
-        return
+        logger.info('no_output is True. Case dump not processed.')
+        return False
 
     if (output_format is None) or (output_format is True):
         output_format = 'xlsx'
 
     output_ext = get_output_ext(output_format)
     if output_ext == '':
-        return
+        return False
 
     if full_path is not None:
         system.files.dump = full_path
@@ -171,7 +172,7 @@ def dump(system, output_format, full_path=None, overwrite=False):
     writer = importlib.import_module('.' + output_format, __name__)
 
     t, _ = elapsed()
-    ret = writer.write(system, system.files.dump, overwrite=overwrite)
+    ret = writer.write(system, system.files.dump, overwrite=overwrite, **kwargs)
     _, s = elapsed(t)
     if ret:
         logger.info(f'Format conversion completed in {s}.')
