@@ -211,10 +211,8 @@ def read(system, file):
             tap = data[2][0]
             phi = data[2][2]
 
-            if tap == 1 and phi == 0:
-                transf = False
-            else:
-                transf = True
+            # set `transf` to True for all entries from `raw['transf']`
+            transf = True
             param = {'bus1': data[0][0], 'bus2': data[0][1], 'u': data[0][11],
                      'b': data[0][8], 'r': data[1][0], 'x': data[1][1],
                      'trans': transf, 'tap': tap, 'phi': phi,
@@ -281,20 +279,20 @@ def read(system, file):
 
 def read_add(system, file):
     """
-    Read addition PSS/E dyr file.
+    Read an addition PSS/E dyr file.
 
     Parameters
     ----------
-    system
-    file
+    system : System
+        System instance to which data will be loaded
+    file : str
+        Path to the additional `dyr` file
 
     Returns
     -------
-
+    bool
+        data parsing status
     """
-
-    warn_experimental("PSS/E dyr support")
-
     with open(file, 'r') as f:
         input_list = [line.strip() for line in f]
 
@@ -316,7 +314,8 @@ def read_add(system, file):
             multi_line = list()
 
     # construct pandas dataframe for all models
-    dyr_dict = dict()
+    dyr_dict = dict()   # input data from dyr file
+
     for psse_model, all_rows in input_concat_dict.items():
         dev_params_num = [([to_number(cell) for cell in row.split()]) for row in all_rows]
         dyr_dict[psse_model] = pd.DataFrame(dev_params_num)
@@ -326,26 +325,64 @@ def read_add(system, file):
     with open(f'{dirname}/psse-dyr.yaml', 'r') as f:
         dyr_yaml = yaml.full_load(f)
 
+    sorted_models = sort_psse_models(dyr_yaml)
+
     for psse_model in dyr_dict:
         if psse_model in dyr_yaml:
             if 'inputs' in dyr_yaml[psse_model]:
                 dyr_dict[psse_model].columns = dyr_yaml[psse_model]['inputs']
 
+    # collect not supported models
+    not_supported = []
+    for model in dyr_dict:
+        if model not in sorted_models:
+            not_supported.append(model)
+
+    # print out debug messages
+    if len(dyr_dict):
+        logger.debug(f'dyr contains models {", ".join(dyr_dict.keys())}')
+
+    if len(not_supported):
+        logger.warning(f'Not supported: {", ".join(not_supported)}')
+    else:
+        logger.debug('All dyr models are supported.')
+
     # load data into models
-    for psse_model in dyr_dict:
+    for psse_model in sorted_models:
+        if psse_model not in dyr_dict:
+            # device not exist
+            continue
+
         if psse_model not in dyr_yaml:
             logger.error(f"PSS/E Model <{psse_model}> is not supported.")
             continue
 
+        logger.debug(f'Parsing PSS/E model {psse_model}')
+
         dest = dyr_yaml[psse_model]['destination']
         find = {}
 
         if 'find' in dyr_yaml[psse_model]:
             for name, source in dyr_yaml[psse_model]['find'].items():
+
                 for model, conditions in source.items():
+                    allow_none = conditions.pop('allow_none', 0)
                     cond_names = conditions.keys()
-                    cond_values = [dyr_dict[psse_model][col] for col in conditions.values()]
-                    find[name] = system.__dict__[model].find_idx(cond_names, cond_values)
+                    cond_values = []
+
+                    for col in conditions.values():
+                        if col in find:
+                            cond_values.append(find[col])
+                        else:
+                            cond_values.append(dyr_dict[psse_model][col])
+
+                    try:
+                        find[name] = system.__dict__[model].find_idx(cond_names, cond_values,
+                                                                     allow_none=allow_none)
+                    except IndexError as e:
+                        logger.error("Data file may contain references to unsupported devices.")
+                        logger.error(e)
+                        return False
 
         if 'get' in dyr_yaml[psse_model]:
             for name, source in dyr_yaml[psse_model]['get'].items():
@@ -384,3 +421,23 @@ def read_add(system, file):
         system.link_ext_param(system.__dict__[dest])
 
     return True
+
+
+def sort_psse_models(dyr_yaml):
+    """
+    Sort supported models so that model names are ordered by dependency.
+    """
+    from andes.models import non_jit
+    from andes.utils.func import list_flatten
+
+    andes_models = list_flatten(list(non_jit.values()))
+    number = dict()
+
+    for psse_model in dyr_yaml:
+        dest = dyr_yaml[psse_model]['destination']
+        if dest in andes_models:
+            number[dest] = andes_models.index(dest)
+
+    sorted_models = [k for k, v in sorted(number.items(), key=lambda item: item[1])]
+
+    return sorted_models
