@@ -5,6 +5,7 @@ from andes.shared import np
 logger = logging.getLogger(__name__)
 
 
+# TODO: offload metadata to YAML file and create classes on the fly.
 class GroupBase(object):
     """
     Base class for groups
@@ -53,10 +54,13 @@ class GroupBase(object):
                            f'<{self._idx2model[idx].class_name}>')
         self._idx2model[idx] = model
 
-    def idx2model(self, idx):
+    def idx2model(self, idx, allow_none=False):
         """
         Find model name for the given idx.
+
+        If `allow_none` is True, will return None at the corr. position.
         """
+
         ret = []
         single = False
         if not isinstance(idx, (list, tuple, np.ndarray)):
@@ -67,7 +71,10 @@ class GroupBase(object):
 
         for i in idx:
             try:
-                ret.append(self._idx2model[i])
+                if i is None and allow_none:
+                    ret.append(None)
+                else:
+                    ret.append(self._idx2model[i])
             except KeyError:
                 raise KeyError(f'Group <{self.class_name}> does not contain idx <{i}>')
 
@@ -75,7 +82,7 @@ class GroupBase(object):
             ret = ret[0]
         return ret
 
-    def get(self, src: str, idx, attr: str = 'v'):
+    def get(self, src: str, idx, attr: str = 'v', allow_none=False, default=0.0):
         """
         Based on the indexer, get the `attr` field of the `src` parameter or variable.
 
@@ -84,9 +91,13 @@ class GroupBase(object):
         src : str
             param or var name
         idx : array-like
-
+            device idx
         attr
             The attribute of the param or var to retrieve
+        allow_none : bool
+            True to allow None values in the indexer
+        default : float
+            If `allow_none` is true, the default value to use for None indexer.
 
         Returns
         -------
@@ -99,19 +110,26 @@ class GroupBase(object):
         if n == 0:
             return np.zeros(0)
 
-        ret = None
-        models = self.idx2model(idx)
+        ret = [''] * n
+        _type_set = False
+        models = self.idx2model(idx, allow_none=allow_none)
+
         for i, idx in enumerate(idx):
-            uid = models[i].idx2uid(idx)
-            instance = models[i].__dict__[src]
-            val = instance.__dict__[attr][uid]
+            if models[i] is not None:
+                uid = models[i].idx2uid(idx)
+                instance = models[i].__dict__[src]
+                val = instance.__dict__[attr][uid]
+            else:
+                val = default
 
             # deduce the type for ret
-            if ret is None:
+            if not _type_set:
                 if isinstance(val, str):
                     ret = [''] * n
                 else:
                     ret = np.zeros(n)
+                _type_set = True
+
             ret[i] = val
 
         return ret
@@ -130,20 +148,30 @@ class GroupBase(object):
             instance = model.__dict__[src]
             instance.__dict__[attr][uid] = value[i]
 
-    def find_idx(self, keys, values):
+    def find_idx(self, keys, values, allow_none=False, default=None):
+        """
+        Find indices of devices that satisfy the given `key=value` condition.
+
+        This method iterates over all models in this group.
+        """
         indices_found = []
+        # `indices_found` contains found indices returned from all models of this group
         for model in self.models.values():
-            indices_found.append(model.find_idx(keys, values, allow_missing=True))
+            indices_found.append(model.find_idx(keys, values, allow_none=True, default=default))
 
         out = []
         for idx, idx_found in enumerate(zip(*indices_found)):
-            if idx_found.count(False) == len(idx_found):
-                missing_values = [item[idx] for item in values]
-                raise IndexError(f'{keys} = {missing_values} not found in {self.class_name}')
+            if not allow_none:
+                if idx_found.count(None) == len(idx_found):
+                    missing_values = [item[idx] for item in values]
+                    raise IndexError(f'{list(keys)} = {missing_values} not found in {self.class_name}')
+
+            real_idx = None
             for item in idx_found:
-                if item is not False:
-                    out.append(item)
+                if item is not None:
+                    real_idx = item
                     break
+            out.append(real_idx)
         return out
 
     def _check_src(self, src: str):
@@ -178,8 +206,7 @@ class GroupBase(object):
                 # name is good
                 pass
             else:
-                logger.warning(f"Group <{self.class_name}>: idx={idx} is used by "
-                               f"{self.idx2model(idx).class_name}. "
+                logger.warning(f"Group {self.class_name}: idx={idx} is used by {self.idx2model(idx).class_name}. "
                                f"Data may be inconsistent.")
                 need_new = True
         else:
@@ -303,7 +330,8 @@ class SynGen(GroupBase):
     def __init__(self):
         super().__init__()
         self.common_params.extend(('Sn', 'Vn', 'fn', 'bus', 'M', 'D'))
-        self.common_vars.extend(('omega', 'delta', 'tm', 'te', 'vf'))
+        self.common_vars.extend(('omega', 'delta', 'tm', 'te', 'vf', 'XadIfd', 'vd', 'vq', 'Id', 'Iq',
+                                 'a', 'v'))
 
 
 class TurbineGov(GroupBase):
