@@ -78,6 +78,33 @@ class PQ(PQData, Model):
                                      ('q2i', 0.0),
                                      ('q2z', 1.0),
                                      )))
+        self.config.add_extra("_help",
+                              pq2z="pq2z conversion if out of voltage limits",
+                              p2p="P constant power percentage for TDS. Must have (p2p+p2i+p2z)=1",
+                              p2i="P constant current percentage",
+                              p2z="P constant impedance percentage",
+                              q2q="Q constant power percentage for TDS. Must have (q2q+q2i+q2z)=1",
+                              q2i="Q constant current percentage",
+                              q2z="Q constant impedance percentage",
+                              )
+        self.config.add_extra("_alt",
+                              pq2z="(0, 1)",
+                              p2p="float",
+                              p2i="float",
+                              p2z="float",
+                              q2q="float",
+                              q2i="float",
+                              q2z="float",
+                              )
+        self.config.add_extra("_tex",
+                              pq2z="z_{pq2z}",
+                              p2p=r"\gamma_{p2p}",
+                              p2i=r"\gamma_{p2i}",
+                              p2z=r"\gamma_{p2z}",
+                              q2q=r"\gamma_{q2q}",
+                              q2i=r"\gamma_{q2i}",
+                              q2z=r"\gamma_{q2z}",
+                              )
 
         self.a = ExtAlgeb(model='Bus',
                           src='a',
@@ -127,7 +154,7 @@ class PQ(PQData, Model):
                                 tex_name='P_{pf}')
         self.Qpf = ConstService(info='Actual Q in power flow',
                                 v_str='(q0 * vcmp_zi + Xlb * vcmp_zl * v0**2 + Xub * vcmp_zu * v0**2)',
-                                tex_name='P_{pf}')
+                                tex_name='Q_{pf}')
         self.Req = ConstService(info='Equivalent resistance at steady state',
                                 v_str='Ppf / v0**2',
                                 tex_name='R_{eq}'
