@@ -13,7 +13,7 @@ class PVData(ModelData):
         super().__init__()
         self.Sn = NumParam(default=100.0, info="Power rating", non_zero=True, tex_name=r'S_n')
         self.Vn = NumParam(default=110.0, info="AC voltage rating", non_zero=True, tex_name=r'V_n')
-        self.subidx = DataParam(info='index for generators on the same bus', export=False, tex_name='idx_{sub}')
+        self.subidx = DataParam(info='index for generators on the same bus', export=False)
         self.bus = IdxParam(model='Bus', info="idx of the installed bus")
         self.busr = IdxParam(model='Bus', info="bus idx for remote voltage control")
         self.p0 = NumParam(default=0.0, info="active power set point in system base", tex_name=r'p_0', unit='p.u.')
@@ -50,7 +50,7 @@ class PVModel(Model):
         super().__init__(system, config)
         self.group = 'StaticGen'
         self.flags.update({'pflow': True,
-                           'collate': True})
+                           })
 
         self.config.add(OrderedDict((('pv2pq', 0),
                                      ('npv2pq', 1))))
@@ -64,6 +64,7 @@ class PVModel(Model):
                               )
         self.config.add_extra("_tex",
                               pv2pq="z_{pv2pq}",
+                              npv2pq="n_{pv2pq}"
                               )
 
         self.a = ExtAlgeb(model='Bus', src='a', indexer=self.bus, tex_name=r'\theta')
@@ -73,6 +74,7 @@ class PVModel(Model):
         self.q = Algeb(info='actual reactive power generation', unit='p.u.', tex_name='q', diag_eps=1e-6)
 
         # TODO: implement switching starting from the second iteration
+        # NOTE: TODO: PV to PQ conversion might not be fully working
         self.qlim = SortedLimiter(u=self.q, lower=self.qmin, upper=self.qmax,
                                   enable=self.config.pv2pq,
                                   n_select=self.config.npv2pq)
@@ -116,7 +118,7 @@ class Slack(SlackData, PVModel):
                               av2pv=(0, 1),
                               )
         self.config.add_extra("_tex",
-                              av2pv="z_{a2pv}",
+                              av2pv="z_{av2pv}",
                               )
         self.a.v_setter = True
         self.a.v_str = 'a0'
