@@ -5,6 +5,7 @@ from collections import OrderedDict
 
 from andes.routines.base import BaseRoutine
 from andes.utils.misc import elapsed, is_notebook
+from andes.utils.tab import Tab
 from andes.shared import tqdm, np
 from andes.shared import matrix, sparse, spdiag
 
@@ -18,7 +19,7 @@ class TDS(BaseRoutine):
     """
     def __init__(self, system=None, config=None):
         super().__init__(system, config)
-        self.config.add(OrderedDict((('tol', 1e-4),
+        self.config.add(OrderedDict((('tol', 1e-6),
                                      ('t0', 0.0),
                                      ('tf', 20.0),
                                      ('fixt', 1),
@@ -48,9 +49,6 @@ class TDS(BaseRoutine):
         if system.options.get('tf') is not None:
             self.config.tf = system.options.get('tf')
 
-        self.tds_models = system.find_models('tds')
-        self.pflow_tds_models = system.find_models(('tds', 'pflow'))
-
         # to be computed
         self.deltat = 0
         self.deltatmin = 0
@@ -59,17 +57,21 @@ class TDS(BaseRoutine):
         self.next_pc = 0
         self.eye = None
         self.Teye = None
+        self.qg = np.array([])
+        self.tol_zero = self.config.tol / 100
 
         # internal status
         self.converged = False
         self.busted = False
+        self.err_msg = ''
         self.niter = 0
-        self._switch_idx = -1  # index into `System.switch_times`
+        self._switch_idx = 0  # index into `System.switch_times`
         self._last_switch_t = -999  # the last critical time
-        self.mis = []
+        self.mis = 1
         self.pbar = None
         self.callpert = None
         self.plotter = None
+        self.plt = None
         self.initialized = False
 
     def init(self):
@@ -84,28 +86,33 @@ class TDS(BaseRoutine):
         """
         t0, _ = elapsed()
         system = self.system
+
+        if self.initialized:
+            return system.dae.xy
+
         self._reset()
         self._load_pert()
-        system.set_address(models=self.tds_models)
-        system.set_dae_names(models=self.tds_models)
+        system.set_address(models=system.exist.tds)
+        system.set_dae_names(models=system.exist.tds)
 
-        system.dae.resize_array()
         system.dae.clear_ts()
-        system.store_sparse_pattern(models=self.pflow_tds_models)
-        system.store_adder_setter(models=self.pflow_tds_models)
+        system.store_sparse_pattern(models=system.exist.pflow_tds)
+        system.store_adder_setter(models=system.exist.pflow_tds)
         system.vars_to_models()
-        system.init(self.tds_models)
-        system.store_switch_times(self.tds_models)
+        system.init(system.exist.tds)
+        system.store_switch_times(system.exist.tds)
         self.eye = spdiag([1] * system.dae.n)
         self.Teye = spdiag(system.dae.Tf.tolist()) * self.eye
+        self.qg = np.zeros(system.dae.n + system.dae.m)
+        self.calc_h()
 
-        self.initialized = self.test_initialization()
+        self.initialized = self.test_init()
         _, s1 = elapsed(t0)
 
         if self.initialized is True:
             logger.info(f"Initialization was successful in {s1}.")
         else:
-            logger.info(f"Initialization failed in {s1}.")
+            logger.error(f"Initialization failed in {s1}.")
 
         if system.dae.n == 0:
             tqdm.write('No dynamic component loaded.')
@@ -123,7 +130,7 @@ class TDS(BaseRoutine):
         out.append('')
         out.append('-> Time Domain Simulation Summary:')
         out.append(f'Sparse Solver: {self.solver.sparselib.upper()}')
-        out.append(f'Simulation time: {self.config.t0}-{self.config.tf}sec.')
+        out.append(f'Simulation time: {self.system.dae.t}-{self.config.tf}sec.')
         if self.config.fixt == 1:
             msg = f'Fixed step size: h={1000 * self.config.tstep:.4g}msec.'
             if self.config.shrinkt == 1:
@@ -135,42 +142,68 @@ class TDS(BaseRoutine):
         out_str = '\n'.join(out)
         logger.info(out_str)
 
-    def run(self, disable_pbar=False, **kwargs):
+    def run(self, no_pbar=False, no_summary=False, **kwargs):
         """
         Run the implicit numerical integration for TDS.
 
         Parameters
         ----------
-        disable_pbar : bool
+        no_pbar : bool
             True to disable progress bar
+        no_summary : bool, optional
+            True to disable the display of summary
         """
         system = self.system
         dae = self.system.dae
         config = self.config
 
-        ret = False
+        succeed = False
+        resume = False
+
         if system.PFlow.converged is False:
             logger.warning('Power flow not solved. Simulation will not continue.')
-            return ret
+            system.exit_code += 1
+            return succeed
 
-        self.summary()
-        self.init()
-        self.pbar = tqdm(total=100, ncols=70, unit='%', file=sys.stdout, disable=disable_pbar)
+        if no_summary is False:
+            self.summary()
+
+        # only initializing at t=0 allows to continue when `run` is called again.
+        if system.dae.t == 0:
+            self.init()
+        else:  # resume simulation
+            resume = True
+
+        self.pbar = tqdm(total=100, ncols=70, unit='%', file=sys.stdout, disable=no_pbar)
+
+        if resume:
+            perc = round((dae.t - config.t0) / (config.tf - config.t0) * 100, 0)
+            self.next_pc = perc + 1
+            self.pbar.update(perc)
 
         t0, _ = elapsed()
-        while (system.dae.t < self.config.tf) and (not self.busted):
-            if self.calc_h() == 0:
-                self.pbar.close()
-                logger.error(f"Simulation terminated at t={system.dae.t:.4f}.")
-                ret = False   # FIXME: overwritten
-                break
 
+        while (system.dae.t < self.config.tf) and (not self.busted):
             if self.callpert is not None:
                 self.callpert(dae.t, system)
 
-            if self._implicit_step():
+            if self._itm_step():  # simulate the current step
                 # store values
-                dae.ts.store_txyz(dae.t, dae.xy, self.system.get_z(models=self.pflow_tds_models))
+                dae.ts.store_txyz(dae.t.tolist(),
+                                  dae.xy,
+                                  self.system.get_z(models=system.exist.pflow_tds),
+                                  )
+                # check if the next step is critical time
+                if self.is_switch_time():
+                    self._last_switch_t = system.switch_times[self._switch_idx]
+                    system.switch_action(system.exist.pflow_tds)
+                    self._switch_idx += 1
+                    system.vars_to_models()
+
+                if self.calc_h() == 0:
+                    logger.error('Time step to zero...')
+                    break
+
                 dae.t += self.h
 
                 # show progress in percentage
@@ -179,25 +212,29 @@ class TDS(BaseRoutine):
                     self.pbar.update(1)
                     self.next_pc += 1
 
-            # check if the next step is critical time
-            if self.is_switch_time():
-                self._last_switch_t = system.switch_times[self._switch_idx]
-                system.switch_action(self.pflow_tds_models)
-                system.vars_to_models()
-
         self.pbar.close()
-        delattr(self, 'pbar')  # removed `pbar` so that System can be dilled
+        delattr(self, 'pbar')  # removed `pbar` so that System object can be dilled
+
+        if self.busted:
+            logger.error(self.err_msg)
+            logger.error(f"Simulation terminated at t={system.dae.t:.4f}.")
+            system.exit_code += 1
+
+        elif system.dae.t == self.config.tf:
+            succeed = True   # success flag
+            system.exit_code += 0
+        else:
+            system.exit_code += 1
 
         _, s1 = elapsed(t0)
         logger.info(f'Simulation completed in {s1}.')
         system.TDS.save_output()
-        ret = True
 
-        # load data into ``TDS.plotter`` in the notebook mode
+        # load data into `TDS.plotter` in the notebook mode
         if is_notebook():
             self.load_plotter()
 
-        return ret
+        return succeed
 
     def rewind(self, t):
         """
@@ -211,34 +248,62 @@ class TDS(BaseRoutine):
         """
         from andes.plot import TDSData  # NOQA
         self.plotter = TDSData(mode='memory', dae=self.system.dae)
+        self.plt = self.plotter
 
-    def test_initialization(self):
+    def test_init(self):
         """
         Update f and g to see if initialization is successful.
         """
         system = self.system
-        system.e_clear(models=self.pflow_tds_models)
-        system.l_update_var(models=self.pflow_tds_models)
-        system.f_update(models=self.pflow_tds_models)
-        system.g_update(models=self.pflow_tds_models)
-        system.l_check_eq(models=self.pflow_tds_models)
-        system.l_set_eq(models=self.pflow_tds_models)
-        system.fg_to_dae()
-        system.j_update(models=self.pflow_tds_models)
+        self._fg_update(system.exist.pflow_tds)
+        system.j_update(models=system.exist.pflow_tds)
+
+        # warn if variables are initialized at limits
+        if system.config.warn_limits:
+            for model in system.exist.pflow_tds.values():
+                for item in model.discrete.values():
+                    item.warn_init_limit()
 
         if np.max(np.abs(system.dae.fg)) < self.config.tol:
             logger.debug('Initialization tests passed.')
             return True
         else:
-            logger.error('Suspect initialization issue!')
+
             fail_idx = np.where(abs(system.dae.fg) >= self.config.tol)
             fail_names = [system.dae.xy_name[int(i)] for i in np.ravel(fail_idx)]
-            logger.error(f"Check variables {', '.join(fail_names)}")
+
+            title = 'Suspect initialization issue! Simulation may crash!'
+            err_data = {'Name': fail_names,
+                        'Var. Value': system.dae.xy[fail_idx],
+                        'Eqn. Mismatch': system.dae.fg[fail_idx],
+                        }
+            tab = Tab(title=title,
+                      header=err_data.keys(),
+                      data=list(map(list, zip(*err_data.values()))))
+
+            logger.error(tab.draw())
+
+            if system.options.get('verbose') == 1:
+                breakpoint()
+            system.exit_code += 1
             return False
 
-    def _implicit_step(self):
+    def _fg_update(self, models):
         """
-        Integrate for a single given step.
+        Update `f` and `g` equations.
+        """
+        system = self.system
+        system.dae.clear_fg()
+        system.l_update_var(models=models)
+        system.s_update_var(models=models)  # update VarService
+        system.f_update(models=models)
+        system.g_update(models=models)
+        system.l_update_eq(models=models)
+        system.fg_to_dae()
+
+    def _itm_step(self):
+        """
+        Integrate with Implicit Trapezoidal Method (ITM) to the current time.
 
         This function has an internal Newton-Raphson loop for algebraized semi-explicit DAE.
         The function returns the convergence status when done but does NOT progress simulation time.
@@ -252,7 +317,7 @@ class TDS(BaseRoutine):
         system = self.system
         dae = self.system.dae
 
-        self.mis = []
+        self.mis = 1
         self.niter = 0
         self.converged = False
 
@@ -261,51 +326,56 @@ class TDS(BaseRoutine):
         self.f0 = np.array(dae.f)
 
         while True:
-            system.e_clear(models=self.pflow_tds_models)
-            system.l_update_var(models=self.pflow_tds_models)
-            system.f_update(models=self.pflow_tds_models)
-            system.g_update(models=self.pflow_tds_models)
-            system.l_check_eq(models=self.pflow_tds_models)
-            system.l_set_eq(models=self.pflow_tds_models)
-            system.fg_to_dae()
+            self._fg_update(models=system.exist.pflow_tds)
 
-            # lazy jacobian update
+            # lazy Jacobian update
             if dae.t == 0 or self.niter > 3 or (dae.t - self._last_switch_t < 0.2):
-                system.j_update(models=self.pflow_tds_models)
+                system.j_update(models=system.exist.pflow_tds)
                 self.solver.factorize = True
 
+            # TODO: set the `Tf` corresponding to the pegged anti-windup limiters to zero.
+            # Although this should not affect anything since corr. mismatches in `self.qg` are reset to zero
+
             # solve implicit trapezoidal method (ITM) integration
             self.Ac = sparse([[self.Teye - self.h * 0.5 * dae.fx, dae.gx],
                               [-self.h * 0.5 * dae.fy, dae.gy]], 'd')
-            # equation `q = 0` is the implicit form of differential equations using ITM
-            q = dae.Tf * (dae.x - self.x0) - self.h * 0.5 * (dae.f + self.f0)
+
+            # equation `self.qg[:dae.n] = 0` is the implicit form of differential equations using ITM
+            self.qg[:dae.n] = dae.Tf * (dae.x - self.x0) - self.h * 0.5 * (dae.f + self.f0)
 
             # reset the corresponding q elements for pegged anti-windup limiter
             for item in system.antiwindups:
-                if len(item.x_set) > 0:
-                    for key, val in item.x_set:
-                        np.put(q, key[np.where(item.zi == 0)], 0)
-            qg = np.hstack((q, dae.g))
+                for key, val in item.x_set:
+                    np.put(self.qg, key, 0)
 
-            inc = self.solver.solve(self.Ac, -matrix(qg))
+            self.qg[dae.n:] = dae.g
+
+            if not self.config.linsolve:
+                inc = self.solver.solve(self.Ac, -matrix(self.qg))
+            else:
+                inc = self.solver.linsolve(self.Ac, -matrix(self.qg))
 
             # check for np.nan first
             if np.isnan(inc).any():
-                logger.error(f'NaN found in solution. Convergence not likely')
+                self.err_msg = 'NaN found in solution. Convergence not likely'
                 self.niter = self.config.max_iter + 1
                 self.busted = True
                 break
 
-            inc[np.where(np.abs(inc) < 1e-12)] = 0  # reset really small values to reduce limiter chattering
+            # reset small values to reduce chattering
+            inc[np.where(np.abs(inc) < self.tol_zero)] = 0
+
             # set new values
-            dae.x += np.ravel(np.array(inc[:dae.n]))
-            dae.y += np.ravel(np.array(inc[dae.n: dae.n + dae.m]))
+            dae.x += inc[:dae.n].ravel()
+            dae.y += inc[dae.n: dae.n + dae.m].ravel()
 
             system.vars_to_models()
 
             # calculate correction
             mis = np.max(np.abs(inc))
-            self.mis.append(mis)
+            if self.niter == 0:
+                self.mis = mis
+
             self.niter += 1
 
             # converged
@@ -315,12 +385,17 @@ class TDS(BaseRoutine):
             # non-convergence cases
             if self.niter > self.config.max_iter:
                 logger.debug(f'Max. iter. {self.config.max_iter} reached for t={dae.t:.6f}, '
-                             f'h={self.h:.6f}, mis={mis:.4g} '
-                             f'({system.dae.xy_name[np.argmax(inc)]})')
+                             f'h={self.h:.6f}, mis={mis:.4g} ')
+
+                # debug helpers
+                g_max = np.argmax(abs(dae.g))
+                inc_max = np.argmax(abs(inc))
+                self._debug_g(g_max)
+                self._debug_ac(inc_max)
+
                 break
-            if mis > 1000 and (mis > 1e8 * self.mis[0]):
-                self.pbar.close()
-                logger.error(f'Error increased too quickly. Convergence not likely.')
+            if mis > 1000 and (mis > 1e8 * self.mis):
+                self.err_msg = 'Error increased too quickly. Convergence not likely.'
                 self.busted = True
                 break
 
@@ -332,6 +407,62 @@ class TDS(BaseRoutine):
 
         return self.converged
 
+    def _debug_g(self, y_idx):
+        """
+        Print out the associated variables with the given algebraic equation index.
+
+        Parameters
+        ----------
+        y_idx
+            Index of the equation into the `g` array. Diff. eqns. are not counted in.
+        """
+        y_idx = y_idx.tolist()
+        logger.debug(f'Max. algebraic mismatch associated with {self.system.dae.y_name[y_idx]} [y_idx={y_idx}]')
+        assoc_vars = self.system.dae.gy[y_idx, :]
+        vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
+
+        logger.debug('')
+        logger.debug(f'{"y_index":<10} {"Variable":<20} {"Derivative":<20}')
+        for v in vars_idx:
+            v = v.tolist()
+            logger.debug(f'{v:<10} {self.system.dae.y_name[v]:<20} {assoc_vars[v]:<20g}')
+
+        pass
+
+    def _debug_ac(self, xy_idx):
+        """
+        Debug Ac matrix by printing out equations and derivatives associated with the max. mismatch variable.
+
+        Parameters
+        ----------
+        xy_idx
+            Index of the maximum mismatch into the `xy` array.
+        """
+
+        xy_idx = xy_idx.tolist()
+        assoc_eqns = self.Ac[:, xy_idx]
+        assoc_vars = self.Ac[xy_idx, :]
+
+        eqns_idx = np.where(np.ravel(matrix(assoc_eqns)))[0]
+        vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
+
+        logger.debug(f'Max. correction is for variable {self.system.dae.xy_name[xy_idx]} [{xy_idx}]')
+        logger.debug(f'Associated equation value is {self.system.dae.fg[xy_idx]:<20g}.')
+        logger.debug('')
+
+        logger.debug(f'{"xy_index":<10} {"Equation":<20} {"Derivative":<20} {"Eq. Mismatch":<20}')
+        for eq in eqns_idx:
+            eq = eq.tolist()
+            logger.debug(f'{eq:<10} {self.system.dae.xy_name[eq]:<20} {assoc_eqns[eq]:<20g} '
+                         f'{self.system.dae.fg[eq]:<20g}')
+
+        logger.debug('')
+        logger.debug(f'{"xy_index":<10} {"Variable":<20} {"Derivative":<20} {"Eq. Mismatch":<20}')
+        for v in vars_idx:
+            v = v.tolist()
+            logger.debug(f'{v:<10} {self.system.dae.xy_name[v]:<20} {assoc_vars[v]:<20g} '
+                         f'{self.system.dae.fg[v]:<20g}')
+
     def save_output(self):
         """
         Save the simulation data into two files: a lst file and a npy file.
@@ -377,9 +508,9 @@ class TDS(BaseRoutine):
         if config.fixt and not config.shrinkt:
             if not self.converged:
                 self.deltat = 0
-                self.pbar.close()
-                logger.error(f"Simulation does not converge for the given step size h={self.config.tstep:.4f}.")
-                logger.error(f"Reduce the step size `tstep`, or set `shrinkt = 1` to let it shrink.")
+                self.busted = True
+                self.err_msg = f"Simulation did not converge with step size h={self.config.tstep:.4f}.\n"
+                self.err_msg += "Reduce the step size `tstep`, or set `shrinkt = 1` to let it shrink."
         else:
             if self.converged:
                 if self.niter >= 15:
@@ -396,8 +527,8 @@ class TDS(BaseRoutine):
                 self.deltat *= 0.9
                 if self.deltat < self.deltatmin:
                     self.deltat = 0
-                    self.pbar.close()
-                    logger.error(f"Time step calculated to zero. Convergence not likely.")
+                    self.err_msg = "Time step reduced to zero. Convergence not likely."
+                    self.busted = True
 
         # last step size
         if system.dae.t + self.deltat > config.tf:
@@ -405,9 +536,9 @@ class TDS(BaseRoutine):
 
         self.h = self.deltat
         # do not skip event switch_times
-        if self._has_more_switch():
-            if (system.dae.t + self.h) > system.switch_times[self._switch_idx + 1]:
-                self.h = system.switch_times[self._switch_idx + 1] - system.dae.t
+        if self._switch_idx < system.n_switches:
+            if (system.dae.t + self.h) > system.switch_times[self._switch_idx]:
+                self.h = system.switch_times[self._switch_idx] - system.dae.t
         return self.h
 
     def _calc_h_first(self):
@@ -450,16 +581,6 @@ class TDS(BaseRoutine):
         self.h = self.deltat
         return self.h
 
-    def _has_more_switch(self):
-        """
-        Check if there are more switching events in the ``System.switch_times`` list.
-        """
-        ret = False
-        if len(self.system.switch_times) > 0:
-            if self._switch_idx + 1 < len(self.system.switch_times):
-                ret = True
-        return ret
-
     def is_switch_time(self):
         """
         Return if the current time is a switching time for time domain simulation.
@@ -472,11 +593,9 @@ class TDS(BaseRoutine):
             ``True`` if is a switching time; ``False`` otherwise.
         """
         ret = False
-        if self._has_more_switch():
-            next_idx = self._switch_idx + 1
-            if abs(self.system.dae.t - self.system.switch_times[next_idx]) < 1e-8:
+        if self._switch_idx < self.system.n_switches:
+            if abs(self.system.dae.t - self.system.switch_times[self._switch_idx]) < 1e-8:
                 ret = True
-                self._switch_idx += 1
         return ret
 
     def _reset(self):
@@ -488,16 +607,18 @@ class TDS(BaseRoutine):
         self.next_pc = 0.1
         self.eye = None
         self.Teye = None
+        self.qg = np.array([])
 
         self.converged = False
         self.busted = False
         self.niter = 0
-        self._switch_idx = -1  # index into `System.switch_times`
+        self._switch_idx = 0       # index into `System.switch_times`
         self._last_switch_t = -999  # the last critical time
-        self.mis = []
-        self.system.dae.t = 0.0
+        self.mis = 1
+        self.system.dae.t = np.array(0.0)
         self.pbar = None
         self.plotter = None
+        self.plt = None             # short name for `plotter`
 
         self.initialized = False
 
