@@ -30,7 +30,7 @@ class Test5Bus(unittest.TestCase):
         self.assertSequenceEqual(self.ss.Bus.idx.v, [0, 1, 2, 3, 4])
         self.assertSequenceEqual(self.ss.Area.idx.v, [1, 2, 3])
 
-    def test_cache_refresn(self):
+    def test_cache_refresh(self):
         self.ss.Bus.cache.refresh()
 
     def test_as_df(self):
@@ -40,14 +40,11 @@ class Test5Bus(unittest.TestCase):
     def test_init_order(self):
         self.ss.Bus.get_init_order()
 
-    def test_pflow(self):
-        self.ss.PFlow.run()
-        self.ss.PFlow.newton_krylov()
-
     def test_pflow_reset(self):
         self.ss.PFlow.run()
-        self.ss.reset()
-        self.ss.PFlow.run()
+        if self.ss.PFlow.config.init_tds == 0:
+            self.ss.reset()
+            self.ss.PFlow.run()
 
     def test_tds_init(self):
         self.ss.PFlow.run()
@@ -60,32 +57,23 @@ class Test5Bus(unittest.TestCase):
         self.ss.PFlow.run()
 
 
-class TestKundur2AreaXLSX(unittest.TestCase):
+class TestKundur2AreaEIG(unittest.TestCase):
     """
     Test Kundur's 2-area system
     """
-    def setUp(self) -> None:
-        xlsx = get_case("kundur/kundur_full.xlsx")
-        self.pss = get_case("kundur/kundur_pss.xlsx")
-        self.ss = andes.run(xlsx)
-
-    def test_xlsx_tds_run(self):
-        self.ss.TDS.config.tf = 10
-        self.ss.TDS.run()
-        andes.main.misc(clean=True)
-
     def test_xlsx_eig_run(self):
+        self.xlsx = get_case("kundur/kundur_full.xlsx")
+        self.ss = andes.run(self.xlsx)
+
         self.ss.EIG.run()
         andes.main.misc(clean=True)
 
-    def test_kundur_pss(self):
-        andes.run(self.pss, routine='tds', no_output=True, tf=10)
-
 
 class TestKundur2AreaPSSE(unittest.TestCase):
     """
-    Test Kundur's 2-area system
+    Test Kundur's 2-area system in PSS/E format
     """
+
     def setUp(self) -> None:
         raw = get_case("kundur/kundur_full.raw")
         dyr = get_case("kundur/kundur_full.dyr")
@@ -95,11 +83,14 @@ class TestKundur2AreaPSSE(unittest.TestCase):
         self.ss_psse.TDS.config.tf = 10
         self.ss_psse.TDS.run()
         andes.main.misc(clean=True)
+        self.assertEqual(self.ss_psse.exit_code, 0, "Exit code is not 0.")
 
     def test_psse_eig_run(self):
         self.ss_psse.EIG.run()
         andes.main.misc(clean=True)
 
+        self.assertEqual(self.ss_psse.exit_code, 0, "Exit code is not 0.")
+
     def test_kundur_psse2xlsx(self):
         output_name = 'test_kundur_convert.xlsx'
         andes.io.dump(self.ss_psse, 'xlsx', full_path=output_name)
@@ -110,12 +101,13 @@ class TestNPCCRAW(unittest.TestCase):
     """
     Test NPCC system in the RAW format.
     """
+
     def test_npcc_raw(self):
-        self.ss = andes.run(get_case('npcc/npcc48.raw'))
+        self.ss = andes.run(get_case('npcc/npcc.raw'))
         andes.main.misc(clean=True)
 
     def test_npcc_raw_tds(self):
-        self.ss = andes.run(get_case('npcc/npcc48.raw'),
+        self.ss = andes.run(get_case('npcc/npcc.raw'),
                             verbose=50,
                             routine='tds',
                             no_output=True,
@@ -128,12 +120,26 @@ class TestNPCCRAW(unittest.TestCase):
         self.ss.dae.print_array('g', tol=1e-4)
 
     def test_npcc_raw_convert(self):
-        self.ss = andes.run(get_case('npcc/npcc48.raw'), convert=True)
+        self.ss = andes.run(get_case('npcc/npcc.raw'), convert=True)
         os.remove(self.ss.files.dump)
+        self.assertEqual(self.ss.exit_code, 0, "Exit code is not 0.")
 
     def test_npcc_raw2json_convert(self):
-        self.ss = andes.run(get_case('npcc/npcc48.raw'),
+        self.ss = andes.run(get_case('npcc/npcc.raw'),
                             convert='json')
-        self.ss2 = andes.run('npcc48.json')
+        self.ss2 = andes.run('npcc.json')
         os.remove(self.ss.files.dump)
         andes.main.misc(clean=True)
+        self.assertEqual(self.ss2.exit_code, 0, "Exit code is not 0.")
+
+
+class TestCOI(unittest.TestCase):
+    def test_kundur_COI(self):
+        ss = get_case('kundur/kundur_coi.xlsx')
+        exit_code = andes.run(ss, routine='tds', no_output=True, tf=0.1, cli=True)
+        self.assertEqual(exit_code, 0, "Exit code is not 0.")
+
+    def test_kundur_COI_empty(self):
+        ss = get_case('kundur/kundur_coi_empty.xlsx')
+        exit_code = andes.run(ss, routine='tds', no_output=True, tf=0.1, cli=True)
+        self.assertEqual(exit_code, 0, "Exit code is not 0.")
